from typing import List

from rest_framework import status
from rest_framework.response import Response


class UnifiedResponse:
    def __init__(
            self,
            data: List[dict] | dict | None = None,
            errors: dict | None = None,
            status_code: int = status.HTTP_200_OK
    ):
        if data is None and errors is None:
            raise ValueError("Data and errors can't be both None")
        self.data = data
        self.errors = errors or []
        self.status_code = status_code

        super().__init__()

    def to_response(self) -> Response:
        return Response(
            {
                'data': self.data,
                'errors': self.errors
            },
            status=self.status_code
        )



from rest_framework.exceptions import NotAuthenticated
from rest_framework.views import exception_handler

from moses.common.exceptions import CustomAPIException


def custom_exception_handler(exc, context):
    response = exception_handler(exc, context)

    if response is not None:
        if isinstance(exc, CustomAPIException):
            response.data = {
                "errors": exc.errors_repr,
                "data": {},
            }
            response.status_code = exc.status_code
        elif isinstance(exc, NotAuthenticated):
            response.data = {
                "errors": {
                    '':[
                        {'error_code': exc.default_code, 'kwargs': {}}
                    ]
                },
                "data": {},
            }
        else:
            response.data = {
                "errors": response.data,
                "data": {},
            }

    return response